%  Table1and2.m
%  
%  This program consists of two parts: PART I is determining no. of breaks for each series;
%  based on the no. of breaks, PART II conducts the subsequent model estimation/selection 
%  and inference. Note the PART I will take around (1000*19 = 19000sec) to run due to bootstrapping; PART II
%  will take 1500sec to run.
%  Note: The p-values generated by the code may be slightly different from
%  those reported in Tables 1 and 2 due to Monte Carlo error (since a fixed seed was not used). 

clc; clear all;
%% PART I: determine no. of breaks for each series
gen_bootre_table1
%% PART II: Table1-column 2&3: break number and break dates
brnum = []; brdate = []; brtime = zeros(19,5); 
load BreakResult_table1.mat;
for j = 1
    tempbrnum = []; tempbrdate = NaN(19,5);
    for i = 1:19
        eval(['total=',' ','br',num2str(i),';']);
        numb = total(1,end); tempbrnum = [tempbrnum;numb];
        if numb ~= 0
            tempbrdate(i,1:numb) = total(numb,1:numb);
        end
    end
    brnum = [brnum, tempbrnum]; brdate = [brdate, tempbrdate];
end
for i = 1:19
    for j = 1:5
        if brdate(i,j) ~= NaN
            dd = brdate(i,j)+1; ddmod = mod(dd,12); ddquo = floor(dd/12);
            if ddmod == 0
                brtime(i,j) = (1960+ddquo-1)*100+12;
            else
                brtime(i,j) = (1960+ddquo)*100+ddmod;
            end
        else
            brtime(i,j) = 0;
        end
    end
end
table1_col2_brnum = brnum;
table1_col3_brdate = brtime;

%%Table1-column 4&5&6&9: mean shift test, Largest AR sum, 90%Band, ADF p-value
load data.csv; col4_meanshift = []; model = []; model_eachbr = [];
err_name = 'err_modelsel'; save(err_name);
for i = 1:19
    cpi = data(:,i); y = 1200*(log(cpi(2:end,1))-log(cpi(1:end-1,1)));
    bigt = length(y)-1; eval(['total=',' ','br',num2str(i),';']);
    numb = total(1,end); brnumi = [numb];
    if numb ~= 0
        estdate = total(numb,1:numb)';
    else
        estdate = [];
    end
    [~, waldresult, lagpure] = robtest_levshift(y,brnumi,estdate);
  
    [re1, re2, re3, re4, sel_model, sel_regimemodel] = robesti_regime(y,estdate,lagpure,waldresult);
    if waldresult == 1
        [re1all, re2all, re3all, re4all] = robesti_regime_each(y,estdate,lagpure,waldresult);
        model_eachbr = [model_eachbr; [re1all, re2all, re3all, re4all]];
    end
	if waldresult == 0 Wre = '  Yes  '; else Wre = '  No   '; end
	[errall, ~] = err_aftersel(y,estdate,lagpure,sel_model,sel_regimemodel,waldresult);
	eval(['err',num2str(i),'=errall;']); save(err_name,strcat(['err',num2str(i)]),'-append'); %%this will be used in making D1andD2
    col4_meanshift = [col4_meanshift; Wre]; model = [model; [re1, re2, re3, re4]];   
end
table1_col4_meanshift = col4_meanshift;
table1_col5_Larsum = round(model(:,1)*100)/100;
table1_col6_90band = round(model(:,11:12)*100)/100;
table1_col9_ADFpvalue = round(model(:,15)*100)/100;
table2 = round([model_eachbr(:,[1,11:12,15])]*100)/100;

%%Table1-column 8: CT test model selection
Pvalue_CT = []; 
for i = 1:19
    %each will take about 60 seconds
    cpi = data(:,i); y = 1200*(log(cpi(2:end,1))-log(cpi(1:end-1,1)));
    [~, pvalue] = CT_test(y); Pvalue_CT = [Pvalue_CT; pvalue];
end
CT_output = [];
for i = 1:19
    CTre=Pvalue_CT(i,:);
    if sum(Pvalue_CT(i,:)==[0 0 0])==3
        CT_output = [CT_output; '  I(0)-I(1)/I(1)-I(0)'];
    else
        [minv, minloc]=min(Pvalue_CT(i,:));
        if minv >=0.1
            CT_output = [CT_output; '  I(0)               '];
        else
            switch minloc
                case 1
                    CT_output = [CT_output; '  I(0)-I(1)          '];
                case 2
                    CT_output = [CT_output; '  I(1)-I(0)          '];
                case 3
                    CT_output = [CT_output; '  I(0)-I(1)/I(1)-I(0)'];
            end
        end
    end
end
countrynames = [ ...
'Austria    '; 'Belgium    '; 'Canada     '; 'Finland    '; ...
'France     '; 'Germany    '; 'Greece     '; 'Italy      '; ...
'Japan      '; 'Korea      '; 'Luxembourg '; 'Netherlands'; ...
'Norway     '; 'Portugal   '; 'Spain      '; 'Sweden     '; ...
'Switzerland'; 'UK         '; 'USA        ';];
table1_col8 = [['CountryName','  CT-test selection  ']; countrynames, CT_output];

%table1_col3_brdate
%[['mshift '; table1_col4_meanshift], table1_col8]
%[table1_col2_brnum, table1_col5_Larsum, table1_col6_90band, table1_col9_ADFpvalue]

table2_names = [ ...
'Belgium    '; 'France     '; 'Germany    '; 
'Italy      '; 'Luxembourg '; ];  %% obtained from previous outputs
%table2

csvwrite('Table1_col3_brdate.csv',table1_col3_brdate);
csvwrite('Table1_col4&8.csv',[['mshift '; table1_col4_meanshift], table1_col8]);
csvwrite('Table1_col2&5&6&9.csv',[table1_col2_brnum, ...
table1_col5_Larsum, table1_col6_90band, table1_col9_ADFpvalue]);
csvwrite('Table2.csv',table2);
disp('Note: Column 7 of Table 1 can be easily deduced from the interval estimation results for each regime in Table 1 and 2');